#ifndef POKEPLATINUM_UNK_020232E0_H
#define POKEPLATINUM_UNK_020232E0_H

#include "struct_defs/struct_02002328.h"
#include "struct_decls/struct_02023350_decl.h"

UnkStruct_02023350 * sub_020232E0(u32 param0, u32 param1, int param2, BOOL param3, u32 param4);
void sub_02023318(UnkStruct_02023350 * param0);
void sub_02023330(UnkStruct_02023350 * param0, int param1, u32 param2);
void sub_020234A0(const UnkStruct_02023350 * param0, u16 param1, UnkStruct_02002328 * param2);
u32 sub_02023620(const UnkStruct_02023350 * param0, const u16 * param1, u32 param2);
BOOL sub_0202366C(const UnkStruct_02023350 * param0, const u16 * param1);
u32 sub_020236D0(const UnkStruct_02023350 * param0, const u16 * param1, u32 param2);
u32 sub_02023738(const UnkStruct_02023350 * param0, const u16 * param1);

#endif // POKEPLATINUM_UNK_020232E0_H
